@@ -43,7 +43,7 @@ namespace Rcpp{
         DataFrame_Impl(SEXP x) : Parent(x) {
             set__(x);
         }
-        DataFrame_Impl( const DataFrame_Impl& other){
+        DataFrame_Impl( const DataFrame_Impl& other) : Parent() {
             set__(other) ;
         }
 
@@ -66,7 +66,7 @@ namespace Rcpp{
         // getAttrib(df, R_RowNamesSymbol) will force an expansion of that
         // compact form thereby allocating a huge vector when we just want
         // the row.names. Hence this workaround.
-        inline int nrows() const {
+        inline int nrow() const {
             SEXP rn = R_NilValue ;
             SEXP att = ATTRIB( Parent::get__() )  ;
             while( att != R_NilValue ){
@@ -83,6 +83,13 @@ namespace Rcpp{
             return LENGTH(rn);
         }
 
+        // Offer multiple variants to accomodate both old interface here and signatures in other classes
+        inline int nrows() const { return DataFrame_Impl::nrow(); }
+        inline int rows()  const { return DataFrame_Impl::nrow(); }
+
+        inline R_xlen_t ncol()  const { return DataFrame_Impl::length(); }
+        inline R_xlen_t cols()  const { return DataFrame_Impl::length(); }
+
         static DataFrame_Impl create(){
             return DataFrame_Impl() ;
         }
@@ -94,7 +101,7 @@ namespace Rcpp{
             if( ::Rf_inherits( x, "data.frame" )){
                 Parent::set__( x ) ;
             } else{
-                SEXP y = internal::convert_using_rfunction( x, "as.data.frame" ) ;
+                Shield<SEXP> y(internal::convert_using_rfunction( x, "as.data.frame" )) ;
                 Parent::set__( y ) ;
             }
         }
@@ -123,9 +130,9 @@ namespace Rcpp{
             obj.erase(strings_as_factors_index) ;
             names.erase(strings_as_factors_index) ;
             obj.attr( "names") = names ;
-            Shield<SEXP> call( Rf_lang3(as_df_symb, obj, wrap( strings_as_factors ) ) ) ;
+            Shield<SEXP> call( Rf_lang3(as_df_symb, obj, Rf_ScalarLogical(strings_as_factors) ) ) ;
             SET_TAG( CDDR(call),  strings_as_factors_symb ) ;
-            Shield<SEXP> res( Rcpp_eval( call ) ) ;
+            Shield<SEXP> res(Rcpp_fast_eval(call, R_GlobalEnv));
             DataFrame_Impl out( res ) ;
             return out ;
 
