@@ -1,7 +1,7 @@
 // headers.h: Rcpp R/C++ interface class library -- R headers
 //
 // Copyright (C) 2008 - 2009 Dirk Eddelbuettel
-// Copyright (C) 2009 - 2013 Dirk Eddelbuettel and Romain Francois
+// Copyright (C) 2009 - 2019 Dirk Eddelbuettel and Romain Francois
 //
 // This file is part of Rcpp.
 //
@@ -26,6 +26,7 @@
 #define R_NO_REMAP
 
 // until September 2019, define RCPP_NO_STRICT_R_HEADERS for transition
+// (that goal was too optimistic, reverse depency checks showed too much breakage)
 #ifndef RCPP_NO_STRICT_R_HEADERS
 # define RCPP_NO_STRICT_R_HEADERS
 #endif
@@ -36,6 +37,13 @@
 # endif
 #endif
 
+// no rtti implies no modules
+#ifdef RCPP_NO_RTTI
+# ifndef RCPP_NO_MODULES
+#  define RCPP_NO_MODULES
+# endif
+#endif
+
 // prevent some macro pollution when including R headers
 // in particular, on Linux, gcc 'leaks' the 'major',
 // 'minor' and 'makedev' macros on Linux; we prevent
