@@ -1,7 +1,7 @@
 #!/usr/bin/env r
 #                     -*- mode: R; ess-indent-level: 4; indent-tabs-mode: nil; -*-
 #
-# Copyright (C) 2010 - 2015  Dirk Eddelbuettel and Romain Francois
+# Copyright (C) 2010 - 2019  Dirk Eddelbuettel and Romain Francois
 #
 # This file is part of Rcpp.
 #
@@ -22,6 +22,9 @@
 
 if (.runThisTest) {
 
+    ## Needed for a change in R 3.6.0 reducing a bias in very large samples
+    suppressWarnings(RNGversion("3.5.0"))
+
     .setUp <- Rcpp:::unitTestSetup("sugar.cpp")
 
     test.sugar.abs <- function( ){
@@ -684,6 +687,36 @@ if (.runThisTest) {
         )
     }
 
+    test.sort_unique <- function() {
+
+        set.seed(123)
+        x <- sample(LETTERS[1:5], 10, TRUE)
+        checkEquals(
+            sort(unique(x), decreasing = TRUE),
+            runit_sort_unique_ch(x, decreasing = TRUE),
+            "unique / character / without NA / decreasing sort"
+        )
+
+        checkEquals(
+            sort(unique(x), decreasing = FALSE),
+            runit_sort_unique_ch(x, decreasing = FALSE),
+            "unique / character / without NA / increasing sort"
+        )
+
+        x <- c(x, NA, NA)
+        checkEquals(
+            sort(unique(x), decreasing = TRUE, na.last = FALSE),
+            runit_sort_unique_ch(x, decreasing = TRUE),
+            "unique / character / with NA / decreasing sort"
+        )
+
+        checkEquals(
+            sort(unique(x), decreasing = FALSE, na.last = TRUE),
+            runit_sort_unique_ch(x, decreasing = FALSE),
+            "unique / character / with NA / increasing sort"
+        )
+    }
+
     test.table <- function(){
         x <- sample( letters, 1000, replace = TRUE )
         checkTrue( all( runit_table(x) == table(x) ) )
@@ -1664,260 +1697,260 @@ if (.runThisTest) {
         )
 
     }
-    
-    
+
+
     ## 10 December 2016
     ## sample.int tests
     test.sugar.sample_dot_int <- function() {
-        
+
         set.seed(123); s1 <- sample_dot_int(10, 5)
         set.seed(123); s2 <- sample(10, 5)
-        
+
         checkEquals(
             s1, s2,
             "sample.int / without replacement / without probability"
         )
-        
+
         set.seed(123); s1 <- sample_dot_int(10, 5, TRUE)
         set.seed(123); s2 <- sample(10, 5, TRUE)
-        
+
         checkEquals(
             s1, s2,
             "sample.int / with replacement / without probability"
         )
-        
+
 
         px <- rep(c(3, 2, 1), length.out = 10)
         set.seed(123); s1 <- sample_dot_int(10, 5, FALSE, px)
         set.seed(123); s2 <- sample(10, 5, FALSE, px)
-        
+
         checkEquals(
             s1, s2,
             "sample.int / without replacement / with probability"
         )
-        
+
         set.seed(123); s1 <- sample_dot_int(10, 5, TRUE, px)
         set.seed(123); s2 <- sample(10, 5, TRUE, px)
-        
+
         checkEquals(
             s1, s2,
             "sample.int / with replacement / with probability"
         )
-        
+
     }
-    
-    
+
+
     ## sample_int tests
     test.sugar.sample_int <- function() {
-        
+
         x <- as.integer(rpois(10, 10))
         px <- rep(c(3, 2, 1), length.out = 10)
-        
+
         set.seed(123); s1 <- sample_int(x, 6)
         set.seed(123); s2 <- sample(x, 6)
-        
+
         checkEquals(
             s1, s2,
             "sample_int / without replacement / without probability"
         )
-        
+
         set.seed(123); s1 <- sample_int(x, 6, TRUE)
         set.seed(123); s2 <- sample(x, 6, TRUE)
-        
+
         checkEquals(
             s1, s2,
             "sample_int / with replacement / without probability"
         )
-        
+
         set.seed(123); s1 <- sample_int(x, 6, FALSE, px)
         set.seed(123); s2 <- sample(x, 6, FALSE, px)
-        
+
         checkEquals(
             s1, s2,
             "sample_int / without replacement / with probability"
         )
-        
+
         set.seed(123); s1 <- sample_int(x, 6, TRUE, px)
         set.seed(123); s2 <- sample(x, 6, TRUE, px)
-        
+
         checkEquals(
             s1, s2,
             "sample_int / with replacement / with probability"
         )
-        
+
     }
-    
+
 
     ## sample_dbl tests
     test.sugar.sample_dbl <- function() {
-        
+
         x <- rnorm(10)
         px <- rep(c(3, 2, 1), length.out = 10)
-        
+
         set.seed(123); s1 <- sample_dbl(x, 6)
         set.seed(123); s2 <- sample(x, 6)
-        
+
         checkEquals(
             s1, s2,
             "sample_dbl / without replacement / without probability"
         )
-        
+
         set.seed(123); s1 <- sample_dbl(x, 6, TRUE)
         set.seed(123); s2 <- sample(x, 6, TRUE)
-        
+
         checkEquals(
             s1, s2,
             "sample_dbl / with replacement / without probability"
         )
-        
+
         set.seed(123); s1 <- sample_dbl(x, 6, FALSE, px)
         set.seed(123); s2 <- sample(x, 6, FALSE, px)
-        
+
         checkEquals(
             s1, s2,
             "sample_dbl / without replacement / with probability"
         )
-        
+
         set.seed(123); s1 <- sample_dbl(x, 6, TRUE, px)
         set.seed(123); s2 <- sample(x, 6, TRUE, px)
-        
+
         checkEquals(
             s1, s2,
             "sample_dbl / with replacement / with probability"
         )
-        
+
     }
-    
+
 
     ## sample_chr tests
     test.sugar.sample_chr <- function() {
-        
+
         x <- sample(letters, 10)
         px <- rep(c(3, 2, 1), length.out = 10)
-        
+
         set.seed(123); s1 <- sample_chr(x, 6)
         set.seed(123); s2 <- sample(x, 6)
-        
+
         checkEquals(
             s1, s2,
             "sample_chr / without replacement / without probability"
         )
-        
+
         set.seed(123); s1 <- sample_chr(x, 6, TRUE)
         set.seed(123); s2 <- sample(x, 6, TRUE)
-        
+
         checkEquals(
             s1, s2,
             "sample_chr / with replacement / without probability"
         )
-        
+
         set.seed(123); s1 <- sample_chr(x, 6, FALSE, px)
         set.seed(123); s2 <- sample(x, 6, FALSE, px)
-        
+
         checkEquals(
             s1, s2,
             "sample_chr / without replacement / with probability"
         )
-        
+
         set.seed(123); s1 <- sample_chr(x, 6, TRUE, px)
         set.seed(123); s2 <- sample(x, 6, TRUE, px)
-        
+
         checkEquals(
             s1, s2,
             "sample_chr / with replacement / with probability"
         )
-        
+
     }
-    
-    
+
+
     ## sample_cx tests
     test.sugar.sample_cx <- function() {
-        
+
         x <- rnorm(10) + 2i
         px <- rep(c(3, 2, 1), length.out = 10)
-        
+
         set.seed(123); s1 <- sample_cx(x, 6)
         set.seed(123); s2 <- sample(x, 6)
-        
+
         checkEquals(
             s1, s2,
             "sample_cx / without replacement / without probability"
         )
-        
+
         set.seed(123); s1 <- sample_cx(x, 6, TRUE)
         set.seed(123); s2 <- sample(x, 6, TRUE)
-        
+
         checkEquals(
             s1, s2,
             "sample_cx / with replacement / without probability"
         )
-        
+
         set.seed(123); s1 <- sample_cx(x, 6, FALSE, px)
         set.seed(123); s2 <- sample(x, 6, FALSE, px)
-        
+
         checkEquals(
             s1, s2,
             "sample_cx / without replacement / with probability"
         )
-        
+
         set.seed(123); s1 <- sample_cx(x, 6, TRUE, px)
         set.seed(123); s2 <- sample(x, 6, TRUE, px)
-        
+
         checkEquals(
             s1, s2,
             "sample_cx / with replacement / with probability"
         )
-        
+
     }
-    
-    
+
+
     ## sample_lgl tests
     test.sugar.sample_lgl <- function() {
-        
+
         x <- rbinom(10, 1, 0.5) > 0
         px <- rep(c(3, 2, 1), length.out = 10)
-        
+
         set.seed(123); s1 <- sample_lgl(x, 6)
         set.seed(123); s2 <- sample(x, 6)
-        
+
         checkEquals(
             s1, s2,
             "sample_lgl / without replacement / without probability"
         )
-        
+
         set.seed(123); s1 <- sample_lgl(x, 6, TRUE)
         set.seed(123); s2 <- sample(x, 6, TRUE)
-        
+
         checkEquals(
             s1, s2,
             "sample_lgl / with replacement / without probability"
         )
-        
+
         set.seed(123); s1 <- sample_lgl(x, 6, FALSE, px)
         set.seed(123); s2 <- sample(x, 6, FALSE, px)
-        
+
         checkEquals(
             s1, s2,
             "sample_lgl / without replacement / with probability"
         )
-        
+
         set.seed(123); s1 <- sample_lgl(x, 6, TRUE, px)
         set.seed(123); s2 <- sample(x, 6, TRUE, px)
-        
+
         checkEquals(
             s1, s2,
             "sample_lgl / with replacement / with probability"
         )
-        
+
     }
-    
-    
+
+
     ## sample_list tests
     test.sugar.sample_list <- function() {
-        
+
         x <- list(
             letters,
-            1:5, 
+            1:5,
             rnorm(10),
             state.abb,
             state.area,
@@ -1928,39 +1961,272 @@ if (.runThisTest) {
             BJsales
         )
         px <- rep(c(3, 2, 1), length.out = 10)
-        
+
         set.seed(123); s1 <- sample_list(x, 6)
         set.seed(123); s2 <- sample(x, 6)
-        
+
         checkEquals(
             s1, s2,
             "sample_list / without replacement / without probability"
         )
-        
+
         set.seed(123); s1 <- sample_list(x, 6, TRUE)
         set.seed(123); s2 <- sample(x, 6, TRUE)
-        
+
         checkEquals(
             s1, s2,
             "sample_list / with replacement / without probability"
         )
-        
+
         set.seed(123); s1 <- sample_list(x, 6, FALSE, px)
         set.seed(123); s2 <- sample(x, 6, FALSE, px)
-        
+
         checkEquals(
             s1, s2,
             "sample_list / without replacement / with probability"
         )
-        
+
         set.seed(123); s1 <- sample_list(x, 6, TRUE, px)
         set.seed(123); s2 <- sample(x, 6, TRUE, px)
-        
+
         checkEquals(
             s1, s2,
             "sample_list / with replacement / with probability"
         )
-        
+
     }
 
+
+    ## 31 January 2017
+    ## upper_tri tests
+    test.sugar.upper_tri <- function() {
+
+        x <- matrix(rnorm(9), 3)
+
+        checkEquals(
+            UpperTri(x), upper.tri(x),
+            "upper_tri / symmetric / diag = FALSE"
+        )
+
+        checkEquals(
+            UpperTri(x, TRUE), upper.tri(x, TRUE),
+            "upper_tri / symmetric / diag = TRUE"
+        )
+
+        x <- matrix(rnorm(12), 3)
+
+        checkEquals(
+            UpperTri(x), upper.tri(x),
+            "upper_tri / [3 x 4] / diag = FALSE"
+        )
+
+        checkEquals(
+            UpperTri(x, TRUE), upper.tri(x, TRUE),
+            "upper_tri / [3 x 4] / diag = TRUE"
+        )
+
+        x <- matrix(rnorm(12), 4)
+
+        checkEquals(
+            UpperTri(x), upper.tri(x),
+            "upper_tri / [4 x 3] / diag = FALSE"
+        )
+
+        checkEquals(
+            UpperTri(x, TRUE), upper.tri(x, TRUE),
+            "upper_tri / [4 x 3] / diag = TRUE"
+        )
+
+    }
+
+
+    ## lower_tri tests
+    test.sugar.lower_tri <- function() {
+
+        x <- matrix(rnorm(9), 3)
+
+        checkEquals(
+            LowerTri(x), lower.tri(x),
+            "lower_tri / symmetric / diag = FALSE"
+        )
+
+        checkEquals(
+            LowerTri(x, TRUE), lower.tri(x, TRUE),
+            "lower_tri / symmetric / diag = TRUE"
+        )
+
+        x <- matrix(rnorm(12), 3)
+
+        checkEquals(
+            LowerTri(x), lower.tri(x),
+            "lower_tri / [3 x 4] / diag = FALSE"
+        )
+
+        checkEquals(
+            LowerTri(x, TRUE), lower.tri(x, TRUE),
+            "lower_tri / [3 x 4] / diag = TRUE"
+        )
+
+        x <- matrix(rnorm(12), 4)
+
+        checkEquals(
+            LowerTri(x), lower.tri(x),
+            "lower_tri / [4 x 3] / diag = FALSE"
+        )
+
+        checkEquals(
+            LowerTri(x, TRUE), lower.tri(x, TRUE),
+            "lower_tri / [4 x 3] / diag = TRUE"
+        )
+
+    }
+
+
+    ## 22 April 2017
+    ## trimws -- vector
+    test.sugar.vtrimws <- function() {
+
+        x <- c(
+            "  a b c", "a b c  ", "  a b c  ",
+            "\t\ta b c", "a b c\t\t", "\t\ta b c\t\t",
+            "\r\ra b c", "a b c\r\r", "\r\ra b c\r\r",
+            "\n\na b c", "a b c\n\n", "\n\na b c\n\n",
+            NA, "", " ", "  \t\r\n  ", "\n \t \r "
+        )
+
+        checkEquals(
+            vtrimws(x), trimws(x),
+            "vtrimws / which = 'both'"
+        )
+
+        checkEquals(
+            vtrimws(x, 'l'), trimws(x, 'l'),
+            "vtrimws / which = 'left'"
+        )
+
+        checkEquals(
+            vtrimws(x, 'r'), trimws(x, 'r'),
+            "vtrimws / which = 'right'"
+        )
+
+        checkException(
+            vtrimws(x, "invalid"),
+            msg = "vtrimws -- bad `which` argument"
+        )
+
+    }
+
+
+    ## trimws -- matrix
+    test.sugar.mtrimws <- function() {
+
+        x <- c(
+            "  a b c", "a b c  ", "  a b c  ",
+            "\t\ta b c", "a b c\t\t", "\t\ta b c\t\t",
+            "\r\ra b c", "a b c\r\r", "\r\ra b c\r\r",
+            "\n\na b c", "a b c\n\n", "\n\na b c\n\n",
+            NA, "", " ", "  \t\r\n  ", "\n \t \r "
+        )
+        x <- matrix(x, nrow = length(x), ncol = 4)
+
+        checkEquals(
+            mtrimws(x), trimws(x),
+            "mtrimws / which = 'both'"
+        )
+
+        checkEquals(
+            mtrimws(x, 'l'), trimws(x, 'l'),
+            "mtrimws / which = 'left'"
+        )
+
+        checkEquals(
+            mtrimws(x, 'r'), trimws(x, 'r'),
+            "mtrimws / which = 'right'"
+        )
+
+        checkException(
+            mtrimws(x, "invalid"),
+            msg = "mtrimws -- bad `which` argument"
+        )
+
+    }
+
+
+    ## trimws -- String
+    test.sugar.strimws <- function() {
+
+        x <- c(
+            "  a b c", "a b c  ", "  a b c  ",
+            "\t\ta b c", "a b c\t\t", "\t\ta b c\t\t",
+            "\r\ra b c", "a b c\r\r", "\r\ra b c\r\r",
+            "\n\na b c", "a b c\n\n", "\n\na b c\n\n",
+            NA, "", " ", "  \t\r\n  ", "\n \t \r "
+        )
+
+        lhs <- vapply(
+            x, strimws, character(1),
+            USE.NAMES = FALSE
+        )
+        rhs <- vapply(
+            x, trimws, character(1),
+            USE.NAMES = FALSE
+        )
+
+        checkEquals(
+            lhs, rhs,
+            "strimws / which = 'both'"
+        )
+
+        lhs <- vapply(
+            x, strimws, character(1),
+            which = 'l', USE.NAMES = FALSE
+        )
+        rhs <- vapply(
+            x, trimws, character(1),
+            which = 'l', USE.NAMES = FALSE
+        )
+
+        checkEquals(
+            lhs, rhs,
+            "strimws / which = 'left'"
+        )
+
+        lhs <- vapply(
+            x, strimws, character(1),
+            which = 'r', USE.NAMES = FALSE
+        )
+        rhs <- vapply(
+            x, trimws, character(1),
+            which = 'r', USE.NAMES = FALSE
+        )
+
+        checkEquals(
+            lhs, rhs,
+            "strimws / which = 'right'"
+        )
+
+        checkException(
+            strimws(x[1], "invalid"),
+            msg = "strimws -- bad `which` argument"
+        )
+
+    }
+
+    ## 21 July 2018
+    ## min/max
+    test.sugar.min.max <- function() {
+        # min(empty) gives NA for integer, Inf for numeric (#844)
+        checkTrue(is.na(intmin(integer(0))),    "min(integer(0))")
+        checkEquals(doublemin(numeric(0)), Inf, "min(numeric(0))")
+
+        # max(empty_ gives NA for integer, Inf for numeric (#844)
+        checkTrue(is.na(intmax(integer(0))),     "max(integer(0))")
+        checkEquals(doublemax(numeric(0)), -Inf, "max(numeric(0))")
+
+        # 'normal' values
+        checkEquals(intmin(c(1:10)),        1L,   "min(integer(...))")
+        checkEquals(doublemin(1.0*c(1:10)), 1.0,  "min(numeric(...))")
+        checkEquals(intmax(c(1:10)),        10L,  "min(integer(...))")
+        checkEquals(doublemax(1.0*c(1:10)), 10.0, "min(numeric(...))")
+    }
 }
